<?php 

namespace Drupal\hubspot_integration\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\hubspot_integration\Services\HubspotAPI;
use Drupal\taxonomy\Entity\Vocabulary;
use Drupal\taxonomy\Entity\Term;

/**
 * Configures Hubspot settings for this site.
 *
 */
class HubspotSortForm extends ConfigFormBase {
  
  /** @var \Drupal\hubspot_integration\Services\HubspotAPI $hubspot_api */
  protected $hubspot_api = NULL;
  
  /** @var array $sort */
  protected $sort;
  
  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('hubspot_integration.api')
    );
  }
  
  /**
   * Constructs a \Drupal\HubspotIntegration\Form\HubspotAdminForm object.
   *
   */
  public function __construct(ConfigFactoryInterface $config_factory, HubspotAPI $hubspot_api) {
    parent::__construct($config_factory);
    $this->hubspot_api = $hubspot_api;
    
    $this->sort = $this->hubspot_api->getSort();
  }
  
  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'hubspot_integration_sort_form';
  }
  
  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['hubspot_integration.settings'];
  }
  
  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('hubspot_integration.settings');
    $mapped_vocs = $this->hubspot_api->getMappedVocabularies();
    
    $form['container'] = [
      '#type' => 'container',
      '#attributes' => ['id' => 'select-container'],
    ];
    
    $form['container']['property_select'] = [
      '#type' => 'select',
      '#title' => 'Select a property',
      '#options' => $mapped_vocs,
      '#default_value' => NULL,
    ];
    
    $form['container']['add_item'] = [
      '#type'   => 'button',
      '#value'  => $this->t('Add another item'),
      '#name' => 'add_item',
      '#ajax'   => [
        'callback' => '::addProperty',
        'wrapper'  => 'sort-container',
      ],
    ];
    
    $form['sort'] = [
      '#type' => 'container',
      '#attributes' => ['id' => 'sort-container'],
    ];
    
    
    foreach($this->sort as $vocId => $sort) {
      $vocabulary = Vocabulary::load($vocId);
      
      $form['sort']['details__'.$vocId] = [
        '#type' => 'details',
        '#title' => $vocabulary->get('name'),
        '#description' => $vocabulary->get('description'),
        '#open' => FALSE,
      ];
      
      $form['sort']['details__'.$vocId]+=$this->sortItemForm($vocabulary, $sort, $form_state);
    }
    
    if ($triggering_element = $form_state->getUserInput()['_triggering_element_name']) {
      
      if ($triggering_element == 'add_item') {
        $vocId = $form_state->getUserInput()['property_select'];
      }
      elseif (stristr($triggering_element, 'vocabulary__')) {
        $vocId = str_replace('vocabulary__', '', $form_state->getUserInput()['_triggering_element_name']);
      }
      
      $vocabulary = Vocabulary::load($vocId);
      
      $form['sort']['details__'.$vocId] = [
        '#type' => 'details',
        '#title' => $vocabulary->get('name'),
        '#description' => $vocabulary->get('description'),
        '#open' => TRUE,
      ];
      
      $form['sort']['details__'.$vocId]+=$this->sortItemForm($vocabulary, NULL, $form_state);
      
      $form_state->setRebuild();
    }
    
    return parent::buildForm($form, $form_state);
  }
  
  /**
   * Build a form item depending on hubspot item type
   */
  protected function sortItemForm(Vocabulary $voc, $sort=NULL, FormStateInterface $form_state) {
    $item = [];
    $terms = \Drupal::entityTypeManager()->getStorage('taxonomy_term')->loadTree($voc->id());
    
    
    /** \Drupal\taxonomy\Entity\Term $term */
    foreach($terms as $term) {
      $item['sort__'.$term->tid] = [
        '#type' => 'details',
        '#title' => $term->name,
        '#attributes' => ['id' => 'sort__'.$term->tid],
      ];
      
      foreach($terms as $weight_term) {
        $item['sort__'.$term->tid]['voc_'.$voc->id().'_term_'.$term->tid.'_weight_'.$weight_term->tid] = [
          '#type' => 'weight',
          '#title' => $weight_term->name,
          '#default_value' => (isset($this->sort[$voc->id()][$term->tid][$weight_term->tid]))?$this->sort[$voc->id()][$term->tid][$weight_term->tid]:0,
        ];
      }
    }
    
    return $item;
  }
  
  /**
   * Ajax callback when add a new mapping item form
   *
   * @param array $form
   * @param FormStateInterface $form_state
   * @return array
   */
  public function addProperty(array &$form, FormStateInterface $form_state) {
    return $form['sort'];
  }
  
  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);
  }
  
  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);
    $this->sort = HubspotSortForm::getSortFromFormState($form_state);
    $config = $this->config('hubspot_integration.settings');
    $config->set('sort', $this->sort);
    $config->save();
  }
  
  protected static function getSortFromFormState(FormStateInterface $form_state) {
    $sort = [];
    //var_dump($form_state->getValues()); 
    foreach($form_state->getValues() as $key => $value) {
      if (preg_match ('/voc_(?P<voc>\w+)_term_(?P<term>\d+)_weight_(?P<weight>\d+)/', $key, $matches)) {
        $vocId = $matches['voc'];
        $tid = $matches['term'];
        $weight_tid = $matches['weight'];
        $sort[$vocId][$tid][$weight_tid]=$value;
      }
    }
    return $sort;
  }
}