<?php

namespace Drupal\hubspot_integration\Plugin\Field\FieldFormatter;

use Drupal\Core\Entity\EntityDisplayRepositoryInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\Plugin\Field\FieldFormatter\EntityReferenceFormatterBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\hubspot_integration\Services\HubspotAPI;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'hubspot_paragraph_formatter' formatter.
 *
 * @FieldFormatter(
 *   id = "hubspot_paragraph_formatter",
 *   label = @Translation("Hubspot Paragraph Rendered Entity"),
 *   field_types = {
 *     "entity_reference_revisions"
 *   }
 * )
 */
class HubspotParagraphsFormatter extends EntityReferenceFormatterBase {
  
  /**
   * The Hubspot API.
   *
   * @var \Drupal\hubspot_integration\Services\HubspotAPI
   */
  protected $hubspot_api;
  
  /**
   * Constructs a StringFormatter instance.
   *
   * @param string $plugin_id
   *   The plugin_id for the formatter.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   The definition of the field to which the formatter is associated.
   * @param array $settings
   *   The formatter settings.
   * @param string $label
   *   The formatter label display setting.
   * @param string $view_mode
   *   The view mode.
   * @param array $third_party_settings
   *   Any third party settings settings.
   * @param LoggerChannelFactoryInterface $logger_factory
   *   The logger factory.
   * @param \Drupal\Core\Entity\EntityDisplayRepositoryInterface $entity_display_repository
   *   The entity display repository.
   * @param \Drupal\hubspot_integration\Services\HubspotAPI
   *   The Hubspot API.
   */
  public function __construct($plugin_id, $plugin_definition, FieldDefinitionInterface $field_definition, array $settings, $label, $view_mode, array $third_party_settings, LoggerChannelFactoryInterface $logger_factory, EntityDisplayRepositoryInterface $entity_display_repository, HubspotAPI $hubspot_api) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $label, $view_mode, $third_party_settings, $logger_factory, $entity_display_repository);
    $this->loggerFactory = $logger_factory;
    $this->entityDisplayRepository = $entity_display_repository;
    $this->hubspot_api = $hubspot_api;
  }
  
  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['label'],
      $configuration['view_mode'],
      $configuration['third_party_settings'],
      $container->get('logger.factory'),
      $container->get('entity_display.repository'),
      $container->get('hubspot_integration.api')
    );
  }
  
  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $view_mode = $this->getSetting('view_mode');
    $elements = array();
    $base_items = $this->getEntitiesToView($items, $langcode);
    
    if ($this->getPluginId() == 'hubspot_paragraph_formatter') {
      //var_dump($base_items);
    }
    
    foreach ($base_items as $delta => $entity) {
      // Protect ourselves from recursive rendering.
      static $depth = 0;
      $depth++;
      if ($depth > 20) {
        $this->loggerFactory->get('entity')->error('Recursive rendering detected when rendering entity @entity_type @entity_id. Aborting rendering.', array('@entity_type' => $entity->getEntityTypeId(), '@entity_id' => $entity->id()));
        return $elements;
      }
      $view_builder = \Drupal::entityTypeManager()->getViewBuilder($entity->getEntityTypeId());
      $elements[$delta] = $view_builder->view($entity, $view_mode, $entity->language()->getId());
      //var_dump($elements[$delta]);
      // Add a resource attribute to set the mapping property's value to the
      // entity's url. Since we don't know what the markup of the entity will
      // be, we shouldn't rely on it for structured data such as RDFa.
      if (!empty($items[$delta]->_attributes) && !$entity->isNew() && $entity->hasLinkTemplate('canonical')) {
        $items[$delta]->_attributes += array('resource' => $entity->toUrl()->toString());
      }
      $depth = 0;
    }
    
    /*var_dump($elements[0]['#paragraph']); 
    var_dump($this->fieldDefinition->getName());
    
    die('etst');*/
    return $elements;
  }
  
  /**
   * {@inheritdoc}
   */
  /*public function viewElements(FieldItemListInterface $items, $langcode) {
    //var_dump($this->getEntitiesToView($items, $langcode)); die;
    
    foreach($this->getEntitiesToView($items, $langcode) as $delta => $item) {
      if ($item->hasField(field_hs_behaviour)) {
        switch ($item->get('field_hs_behaviour')->getString()) {
          case 'filtering':
            //var_dump($items);
            //var_dump($item->get('field_hs_behaviour')->getString());
            break;
          case 'highlight':
            break;
          default:
              
        }
      }
    }
    
    $view_mode = $this->getSetting('view_mode');
    $elements = array();
    
    foreach ($this->getEntitiesToView($items, $langcode) as $delta => $entity) {
      // Protect ourselves from recursive rendering.
      static $depth = 0;
      $depth++;
      if ($depth > 20) {
        $this->loggerFactory->get('entity')->error('Recursive rendering detected when rendering entity @entity_type @entity_id. Aborting rendering.', array('@entity_type' => $entity->getEntityTypeId(), '@entity_id' => $entity->id()));
        return $elements;
      }
      $view_builder = \Drupal::entityTypeManager()->getViewBuilder($entity->getEntityTypeId());
      $elements[$delta] = $view_builder->view($entity, $view_mode, $entity->language()->getId());
      
      // Add a resource attribute to set the mapping property's value to the
      // entity's url. Since we don't know what the markup of the entity will
      // be, we shouldn't rely on it for structured data such as RDFa.
      if (!empty($items[$delta]->_attributes) && !$entity->isNew() && $entity->hasLinkTemplate('canonical')) {
        $items[$delta]->_attributes += array('resource' => $entity->toUrl()->toString());
      }
      $depth = 0;
    }
    var_dump($elements);
    die;
    return $elements;
    
  }*/
  
  

  /**
   * {@inheritdoc}
   */
  public static function isApplicable(FieldDefinitionInterface $field_definition) {
    return parent::isApplicable($field_definition);
  }
  
  public static function getHubspotModeOptions() {
    return array(
      'default' => 'Default',
      'highlight' => 'Highlighting',
      'filtering' => 'Filtering with callback',
    );
  }
  
  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    $defaults = parent::defaultSettings();
    
    $defaults['hs_behaviour'] = 'default';
    
    return $defaults;
  }
  
  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $elements = parent::settingsForm($form, $form_state);
    
    $elements['hs_behaviour'] = array(
      '#type' => 'radios',
      '#options' => $this->getHubspotModeOptions(),
      '#title' => $this->t('Default Hubspot behaviour'),
      '#default_value' => $this->getSetting('hs_behaviour'),
      '#required' => TRUE,
    );
    
    return $elements;
  }
  
  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = parent::settingsSummary();
    $hubspot_behaviours = $this->getHubspotModeOptions();
    $hubspot_behaviour = $this->getSetting('hs_behaviour');
    $summary[] = $this->t('Default Hubspot behaviour : @hs_behaviour', array('@hs_behaviour' => isset($hubspot_behaviours[$hubspot_behaviour]) ? $hubspot_behaviours[$hubspot_behaviour] : $hubspot_behaviour));
    
    return $summary;
  }
  
}
