<?php

namespace Drupal\hubspot_integration\Plugin\Field\FieldWidget;

use Drupal\Component\Utility\NestedArray;
use Drupal\Component\Utility\Html;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\Entity\EntityFormDisplay;
use Drupal\Core\Entity\FieldableEntityInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldFilteredMarkup;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\SubformState;
use Drupal\Core\Render\Element;
use Drupal\Core\TypedData\TranslationStatusInterface;
use Drupal\paragraphs\ParagraphInterface;
use Drupal\paragraphs\Plugin\Field\FieldWidget\ParagraphsWidget;
use Drupal\paragraphs\Plugin\EntityReferenceSelection\ParagraphSelection;
use Symfony\Component\Validator\ConstraintViolationInterface;
use Symfony\Component\Validator\ConstraintViolationListInterface;

/**
 * Plugin implementation of the 'hubspot_paragraphs_widget' widget.
 *
 * @FieldWidget(
 *   id = "hubspot_paragraphs_widget",
 *   module = "hubspot_integration",
 *   label = @Translation("Hubspot Paragraph widget"),
 *   field_types = {
 *     "entity_reference_revisions"
 *   }
 * )
 */
class HubspotParagraphsWidget extends ParagraphsWidget {
  
  /**
   * Action position is in the add paragraphs place.
   */
  const ACTION_POSITION_BASE = 1;
  
  /**
   * Action position is in the table header section.
   */
  const ACTION_POSITION_HEADER = 2;
  
  /**
   * Action position is in the actions section of the widget.
   */
  const ACTION_POSITION_ACTIONS = 3;
  
  /**
   * Indicates whether the current widget instance is in translation.
   *
   * @var bool
   */
  protected $isTranslating;
  
  /**
   * Id to name ajax buttons that includes field parents and field name.
   *
   * @var string
   */
  protected $fieldIdPrefix;
  
  /**
   * Wrapper id to identify the paragraphs.
   *
   * @var string
   */
  protected $fieldWrapperId;
  
  /**
   * Number of paragraphs item on form.
   *
   * @var int
   */
  protected $realItemCount;
  
  /**
   * Parents for the current paragraph.
   *
   * @var array
   */
  protected $fieldParents;
  
  /**
   * Accessible paragraphs types.
   *
   * @var array
   */
  protected $accessOptions = NULL;
  
  /**
   * Constructs a ParagraphsWidget object.
   *
   * @param string $plugin_id
   *   The plugin_id for the widget.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   The definition of the field to which the widget is associated.
   * @param array $settings
   *   The widget settings.
   * @param array $third_party_settings
   *   Any third party settings.
   */
  public function __construct($plugin_id, $plugin_definition, FieldDefinitionInterface $field_definition, array $settings, array $third_party_settings) {
    // Modify settings that were set before https://www.drupal.org/node/2896115.
    if(isset($settings['edit_mode']) && $settings['edit_mode'] === 'preview') {
      $settings['edit_mode'] = 'closed';
      $settings['closed_mode'] = 'preview';
    }
    
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $third_party_settings);
  }
  
  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    $defaults = parent::defaultSettings();
    
    $defaults['hs_wrapper'] = 0;
    $defaults['hs_behaviour'] = 'default';
    
    return $defaults;
  }
  
  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $elements = parent::settingsForm($form, $form_state);
    
    $elements['hs_wrapper'] = array(
      '#type' => 'radios',
      '#options' => $this->getSettingOptions('hs_wrapper'),
      '#title' => $this->t('Hubspot Wrapper'),
      '#default_value' => $this->getSetting('hs_wrapper'),
      '#required' => FALSE,
    );
    
    $elements['hs_behaviour'] = array(
      '#type' => 'radios',
      '#options' => $this->getSettingOptions('hs_behaviour'),
      '#title' => $this->t('Default Hubspot behaviour'),
      '#default_value' => $this->getSetting('hs_behaviour'),
      '#required' => TRUE,
    );
    
    return $elements;
  }
  
  /**
   * Returns select options for a plugin setting.
   *
   * This is done to allow
   * \Drupal\paragraphs\Plugin\Field\FieldWidget\ParagraphsWidget::settingsSummary()
   * to access option labels. Not all plugin setting are available.
   *
   * @param string $setting_name
   *   The name of the widget setting. Supported settings:
   *   - "edit_mode"
   *   - "closed_mode"
   *   - "autocollapse"
   *   - "add_mode"
   *
   * @return array|null
   *   An array of setting option usable as a value for a "#options" key.
   */
  protected function getSettingOptions($setting_name) {
    $options = parent::getSettingOptions($setting_name);
    
    if ($setting_name == 'hs_wrapper') {
      $options = [
        0 => 'No',
        1 => 'Yes',
      ];
    }
    elseif ($setting_name == 'hs_behaviour') {
      $options = [
        'default' => 'Default',
        'highlight' => 'Highlighting',
        'filtering' => 'Filtering with callback',
      ];
    }
    
    
    return !empty($options) ? $options : NULL;
  }
  
  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = parent::settingsSummary();
    
    $summary[] = $this->t('Hubspot wrapper: @hs_wrapper', ['@hs_wrapper' => $this->getSetting('hs_wrapper')]);
    $summary[] = $this->t('Hubspot Behaviour: @hs_behaviour', ['@hs_behaviour' => $this->getSetting('hs_behaviour')]);
    
    return $summary;
  }
  
  /**
   * {@inheritdoc}
   *
   * @see \Drupal\content_translation\Controller\ContentTranslationController::prepareTranslation()
   *   Uses a similar approach to populate a new translation.
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $element = parent::formElement($items, $delta, $element, $form, $form_state);
    return $element;
  }
  
  public static function getHubspotModeOptions() {
    return array(
      'default' => 'Default',
      'highlight' => 'Highlighting',
      'filtering' => 'Filtering with callback',
    );
  }
  
  /**
   * {@inheritdoc}
   */
  public function form(FieldItemListInterface $items, array &$form, FormStateInterface $form_state, $get_delta = NULL) {
    $elements = [];
    
    $elements['hs_behaviour'] = array(
      '#type' => 'radios',
      '#options' => $this->getHubspotModeOptions(),
      '#title' => $this->t('Hubspot behaviour'),
      '#default_value' => $this->getSetting('hs_behaviour'),
      '#required' => TRUE,
    );
    
    return $elements + parent::form($items, $form, $form_state, $get_delta);
  }
  
}