<?php

namespace Drupal\hubspot_integration\Plugin\views\sort;

use Drupal\Core\Cache\CacheableDependencyInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\hubspot_integration\Services\HubspotAPI;
use Drupal\taxonomy\Entity\Term;
use Drupal\views\Plugin\views\sort\SortPluginBase;
use Drupal\views\Plugin\ViewsHandlerManager;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Handle a Hubspot term sort.
 *
 * @ViewsSort("hs_persona")
 */
class HubspotTerms extends SortPluginBase implements CacheableDependencyInterface {

  /**
   * The hubspot api.
   *
   * @var \Drupal\hubspot_integration\Services\HubspotAPI
   */
  protected $hubspot_api;

  /**
   * The views handler manager.
   *
   * @var \Drupal\views\Plugin\ViewsHandlerManager
   */
  protected $joinHandler;

  /**
   * Constructs a Handler object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\hubspot_integration\Services\HubspotAPI $hubspot_api
   *   The Hubspot API
   * @param \Drupal\views\Plugin\ViewsHandlerManager $join_handler
   *   The views handler manager.
   */
  public function __construct(array $configuration,
                              $plugin_id,
                              $plugin_definition,
                              HubspotAPI $hubspot_api,
                              ViewsHandlerManager $join_handler) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->hubspot_api = $hubspot_api;
    $this->is_handler = TRUE;
    $this->joinHandler = $join_handler;
  }

  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('hubspot_integration.api'),
      $container->get('plugin.manager.views.join')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function usesGroupBy() {
    return FALSE;
  }

  public function query() {
    if (($cookie = $this->hubspot_api->getCookie('hubspot_integration'))
      && ($tids = explode('_', ltrim($cookie, 'hubspot_')))
      && ($sort = $this->hubspot_api->getSort())
    ) {
      if (!empty($tids[0])) {
        $tid = $tids[0];

        /** @var \Drupal\taxonomy\Entity\Term $term */
        $term = Term::load($tid);

        $vid = $term->getVocabularyId();
        $when = '';

        if (!empty($sort[$vid][$tid])) {
          foreach ($sort[$vid][$tid] as $term_weight_id => $term_weight) {
            $when .= sprintf('WHEN {hubspot_sort_node__hs_persona}.hs_persona_target_id=%d THEN %d ', $term_weight_id, $term_weight);
          }
        }

        if (!empty($when)) {
          $definition = [
            'table' => 'node__hs_persona',
            'type' => 'LEFT',
            'field' => 'entity_id',
            'left_table' => 'node_field_data',
            'left_field' => 'nid',
            'extra' => 'hubspot_sort_node__hs_persona.hs_persona_target_id = ' . $tid,
          ];
          $join = $this->joinHandler->createInstance('standard', $definition);
          $rel = $this->query->addRelationship('hubspot_sort_node__hs_persona', $join, 'node_field_data');
          $this->query->addTable('node__hs_persona', $rel, $join);

          $alias = $this->realField;
          $this->query->addOrderBy(NULL, "CASE $when ELSE 999 END", $this->options['order'], $alias);
        }
      }
    }
  }

  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    parent::buildOptionsForm($form, $form_state);
  }

  /**
  +   * {@inheritdoc}
  +   */
  public function getCacheContexts() {
      return ['cookies:hubspot_integration'];
  }
}
