<?php

namespace Drupal\hubspot_integration\Services;

use Drupal\Component\Utility\UrlHelper;
use Drupal\Core\Database\Connection;
use Symfony\Component\HttpFoundation\Session\Session;
use Exception;
use Drupal\Core\Config\ConfigFactory;
use Drupal\taxonomy\Entity\Term;
use Drupal\taxonomy\Entity\Vocabulary;

class HubspotAPI {
  /*
   * @var \Drupal\Core\Database\Connection $database
   */
  protected $database;
  /*
   * @var \Symfony\Component\HttpFoundation\Session\Session $session
   */
  protected $session;
  /*
   * @var \Drupal\Core\Config\ConfigFactory $config_factory
   */
  protected $config_factory;

  /**
   * Static cache for cookies.
   *
   * @var array
   */
  protected $cookies;

  /**
   * Constructs a new HubspotAPI object.
   * @param \Drupal\Core\Database\Connection $connection
   * @param \Symfony\Component\HttpFoundation\Session\Session $session
   * @param \Drupal\Core\Config\ConfigFactory $config_factory
   */
  public function __construct(Connection $connection, Session $session, ConfigFactory $config_factory) {
    $this->database = $connection;
    $this->session = $session;
    $this->config_factory = $config_factory;
  }

  /**
   * Get Hubspot cookies and perform security check when necessary.
   *
   * @param string $cookieName
   *   The cookie to retrieve.
   *
   * @return string
   *   The sanitized cookie value.
   */
  public function getCookie($cookieName = 'hubspotutk') {
    if (!isset($this->cookies[$cookieName])) {
      $this->cookies[$cookieName] = \Drupal::request()->cookies->get($cookieName);

      // Ensure data in cookie is authorized / safe for hubspot_integration.
      if (($cookieName == 'hubspot_integration')
        && ($tids = explode('_', ltrim($this->cookies[$cookieName], 'hubspot_')))
        && ($mapping = $this->getMapping())
      ) {
        $allowedValues = [];
        foreach ($mapping as $map) {
          foreach ($map['#terms'] as $termId) {
            if (!empty($termId)) {
              $allowedValues[] = $termId;
            }
          }
        }

        $tids = array_intersect($tids, $allowedValues);
        $this->cookies[$cookieName] = 'hubspot_' . implode('_', $tids);
      }
    }

    return $this->cookies[$cookieName];
  }

  /**
   * Get current session
   */
  public function getSession() {
    return $this->session;
  }

  /**
   * Determine if current user is registered as a contact on husbpot.
   *
   * @return bool
   *   Either or not the user is a contact.
   */
  public function isContact() {
    $result = FALSE;

    if ($user = $this->getContactInfo()) {
      $property = 'is-contact';

      if (isset($user->{$property})) {
        $result = $user->{$property};
      }
    }

    return $result;
  }

  /**
   * Get Husbspot values for the current user
   *
   * @param boolean $forced_update
   * @return NULL|mixed
   */
  public function getContactInfo($forced_update=false) {
    $user = NULL;
    if (!$this->isDemoMode() && !$forced_update && $this->session->get('hubspot_integration.user')) {
      $user = $this->session->get('hubspot_integration.user');
    }
    else {
      $cookie = $this->getCookie();
      if(!empty($cookie)) {
        $user = $this->request("contacts/v1/contact/utk/$cookie/profile");
      }
    }
    return $user;
  }

  /**
   * Get the config name for the hubspot integration module settings
   *
   * @param unknown $name
   * @return mixed|NULL|array|\Drupal\Component\Render\MarkupInterface|string|unknown[]|array[]
   */
  public function getConfig($name) {
    return $this->config_factory->get('hubspot_integration.settings')->get($name);
  }

  /**
   * Set the config name for the hubspot integration module settings
   *
   * @param unknown $name
   * @return mixed|NULL|array|\Drupal\Component\Render\MarkupInterface|string|unknown[]|array[]
   */
  public function setConfig($name, $value) {
    return $this->config_factory->getEditable('hubspot_integration.settings')->set($name, $value)->save();
  }

  /**
   * Get the list of the possible contact properties and their definitions
   *
   * @return mixed
   */
  public function getContactProperties() {
    return $this->request('properties/v1/contacts/properties');
  }

  /**
   * Function to make calls on hubspot
   *
   * @param unknown $api
   * @return mixed
   */
  public function request($api, $options = [], $type='GET', $datas = []) {
    $api_key = $this->getConfig('api_key');
    $ret = NULL;
    if (!empty($api_key)) {
      $hub_url = 'https://api.hubapi.com/';
      $options = ['hapikey'=>$api_key]+$options;
      $query_str = UrlHelper::buildQuery($options);
      $url = $hub_url.$api.'?'.$query_str;

      try {
        switch($type) {
          case 'GET':
            $request = \Drupal::httpClient()->get($url, []);
            break;
          case 'POST':
            $request = \Drupal::httpClient()->post($url, ['json' => $datas], $options);
            break;
        }
        $ret = json_decode($request->getBody());
      }
      catch (Exception $e) {
        \Drupal::logger('hubspot_integration')->error('Error : '.$e->getCode(). ' : ' .$e->getMessage());
      }
    }
    return $ret;
  }

  /**
   * Get the mapping done between Drupal and Hubspot
   * @return mixed
   */
  public function getMapping() {
    return $this->getConfig('mapping');;
  }

  /**
   * Get the sort configuration
   * @return mixed
   */
  public function getSort() {
    return $this->getConfig('sort');;
  }

  /**
   * Utility function to transform a text as a machine name
   *
   * @param unknown $string
   * @return string
   */
  public static function machineName($string) {
    return preg_replace('@[^a-z0-9-]+@','-', strtolower($string));
  }

  /**
   * Utility function to check a vocabulary is mapped with hubspot
   * @param string $vocabulary
   * @return boolean
   */
  public function isVocabularyMapped($vocabulary) {
    $mapping = $this->getMapping();

    foreach($mapping as $mapping) {
      if (isset($mapping['#vocabulary']) && $mapping['#vocabulary']==$vocabulary) {
        return true;
      }
    }

    return false;
  }

  /**
   * Utility function to get mapped vocabularies
   * @return array
   */
  function getMappedVocabularies() {
    $vocabularies = Vocabulary::loadMultiple();
    $vocabulariesList = [];

    foreach ($vocabularies as $vid => $vocablary) {
      if ($this->isVocabularyMapped($vid)) {
        $vocabulariesList[$vid] = $vocablary->get('name');
      }
    }

    return $vocabulariesList;
  }

  public function isDemoMode() {
    return $this->getConfig('demo_mode');
  }

  public function anonymousContactCreationEnabled() {
    return $this->getConfig('anonymous_contact');
  }

  public function demoMessage($message) {
    if ($this->isDemoMode()) {
      \Drupal::messenger()->addStatus("Demo Hubspot : $message");
    }
  }

  public function getUserTids() {
    $tids = [];

    if (($hub_user = $this->getContactInfo())
      && ($mapping = $this->getMapping())
      && !empty($mapping)
    ) {
      $taxonomy = [];

      foreach ($mapping as $hub_voc => $map) {
        if (isset($hub_user->properties->{$hub_voc})) {
          foreach ($map['#terms'] as $hub_term_id => $taxonomy_term_id) {
            if ($taxonomy_term_id && isset($hub_user->properties->{$hub_voc}->value) && HubspotAPI::machineName($hub_user->properties->{$hub_voc}->value) == $hub_term_id) {
              $taxonomy_term = Term::load($taxonomy_term_id);
              $taxonomy[$taxonomy_term->id()] = $taxonomy_term->bundle();
            }
          }
        }
      }

      foreach ($taxonomy as $tid => $vocab) {
        $tids[] = $tid;
      }
    }

    return $tids;
  }

  public function getAnonymousContactNumber($update = false) {
    if ($update == true) {
      $request = $this->request('contacts/v1/search/query', ['q'=>$this->getAnonymousContactMailTemplate('*')]);
      if (!empty($request)) {
        $this->setConfig('anonymous_contacts_count', $request->total);
        return $request->total;
      }
      else {
        return 0;
      }
    }
    else {
      return $this->getConfig('anonymous_contacts_count');
    }
  }

  public function getAnonymousContactMailTemplate($hub_cookie=NULL) {
    $template = $this->getConfig('anonymous_mail_template');

    if (!empty($hub_cookie)) {
      $template = str_replace ( '[hub_cookie]' , $hub_cookie , $template);
    }
    return $template;
  }
}
